@@ -16,7 +16,7 @@ extern sqlite3 *db;
 
 void create_columns_exp(List*, char*, char*);
 
-static List* db_get_element_types(int all, List* elements)
+static List* db_get_element_types(int all, List* element_types)
 {
     int r;
     const char *sql;
@@ -32,30 +32,30 @@ static List* db_get_element_types(int all, List* elements)
 
     while (SQLITE_ROW == (r = sqlite3_step(stmt))){
         unsigned const char* value;
-        ElementType* e = list_new_element(elements);
+        ElementType* et = list_new_element(element_types);
 
-        e->id = sqlite3_column_int(stmt, 0);
-        e->type = sqlite3_column_int(stmt, 1);
-        e->ticket_property = sqlite3_column_int(stmt, 2);
-        e->reply_property = sqlite3_column_int(stmt, 3);
-        e->required = sqlite3_column_int(stmt, 4);
-        strcpy(e->name, sqlite3_column_text(stmt, 5));
-        strcpy(e->description, sqlite3_column_text(stmt, 6));
-        e->display_in_list = sqlite3_column_int(stmt, 7);
-        e->sort = sqlite3_column_int(stmt, 8);
+        et->id = sqlite3_column_int(stmt, 0);
+        et->type = sqlite3_column_int(stmt, 1);
+        et->ticket_property = sqlite3_column_int(stmt, 2);
+        et->reply_property = sqlite3_column_int(stmt, 3);
+        et->required = sqlite3_column_int(stmt, 4);
+        strcpy(et->name, sqlite3_column_text(stmt, 5));
+        strcpy(et->description, sqlite3_column_text(stmt, 6));
+        et->display_in_list = sqlite3_column_int(stmt, 7);
+        et->sort = sqlite3_column_int(stmt, 8);
         value = sqlite3_column_text(stmt, 9);
         if (value != NULL) {
-            strcpy(e->default_value, value);
+            strcpy(et->default_value, value);
         }
-        e->auto_add_item = sqlite3_column_int(stmt, 10);
-        list_add(elements, e);
+        et->auto_add_item = sqlite3_column_int(stmt, 10);
+        list_add(element_types, et);
     }
     if (SQLITE_DONE != r)
         goto error;
 
     sqlite3_finalize(stmt);
 
-    return elements;
+    return element_types;
 
 ERROR_LABEL
 }
@@ -106,7 +106,7 @@ ElementType* db_get_element_type(int id, ElementType* e)
 ERROR_LABEL
 }
 
-List* db_get_list_item(int element_type, List* items)
+List* db_get_list_item(const int element_type, List* items)
 {
     int r;
     const char *sql = "select id, name, close, sort from list_item where element_type_id = ? order by sort";
@@ -167,7 +167,8 @@ int db_register_ticket(Message* ticket)
     List* elements;
     Iterator* it;
     char sql[DEFAULT_LENGTH];
-    int i, closed = 0;
+    int i;
+    bool closed = false;
     sqlite3_stmt *stmt = NULL;
     int message_id;
     int register_mode = ticket->id == -1;
@@ -193,7 +194,7 @@ int db_register_ticket(Message* ticket)
                 COLUMN_TYPE_TEXT, e->str_val,
                 COLUMN_TYPE_END);
         if (c != INVALID_INT && c != 0) {
-            closed = 1;
+            closed = true;
             break;
         }
     }
@@ -262,8 +263,7 @@ int db_register_ticket(Message* ticket)
                     COLUMN_TYPE_BLOB, content_a,
                     COLUMN_TYPE_END) == 0)
                 die("insert failed.");
-            xfree(content_a->blob);
-            xfree(content_a);
+            element_file_free(content_a);
         }
     }
     return ticket->id;
@@ -318,24 +318,25 @@ static String* get_search_sql_string(List* conditions, Condition* sort, List* ke
     int i = 0;
     Iterator* it;
 
-    string_append(sql_string,
+    string_appendf(sql_string,
             "select "
-            " t.id "
+            " t.id as id, m.field%d as state "
             "from ticket as t "
             "inner join message as m on m.id = t.last_message_id "
-            "inner join message as org_m on org_m.id = t.original_message_id ");
+            "inner join message as org_m on org_m.id = t.original_message_id ", ELEM_ID_STATUS);
 
     
     if (keywords->size > 0)
         string_append(sql_string, "inner join message as m_all on m_all.ticket_id = t.id ");
 
-    if (conditions->size || keywords->size > 0)
+    if (valid_condition_size(conditions) || keywords->size > 0)
         string_append(sql_string, "where ");
-    if (conditions->size) {
+    if (valid_condition_size(conditions)) {
         foreach (it, conditions) {
             Condition* cond = it->element;
             char val[DEFAULT_LENGTH];
             if (cond->element_type_id < 0) continue;
+            if (strlen(cond->value) == 0) continue;
             if (i++) string_append(sql_string, " and ");
             switch (cond->condition_type) {
                 case CONDITION_TYPE_DATE_FROM:
@@ -357,6 +358,7 @@ static String* get_search_sql_string(List* conditions, Condition* sort, List* ke
             char name[DEFAULT_LENGTH];
             char val[DEFAULT_LENGTH];
             if (cond->element_type_id > 0) continue;
+            if (strlen(cond->value) == 0) continue;
             if (i++) string_append(sql_string, " and ");
             switch (cond->element_type_id) {
                 case ELEM_ID_REGISTERDATE:
@@ -383,7 +385,7 @@ static String* get_search_sql_string(List* conditions, Condition* sort, List* ke
         list_alloc(element_types_a, ElementType);
         element_types_a = db_get_element_types_all(element_types_a);
         columns_a = create_columns_like_exp(element_types_a, "m_all", keywords, columns_a);
-        if (conditions->size)
+        if (valid_condition_size(conditions))
             string_append(sql_string, " and ");
         string_appendf(sql_string, "(%s)", string_rawstr(columns_a));
         string_free(columns_a);
@@ -426,6 +428,7 @@ int set_conditions(sqlite3_stmt* stmt, List* conditions, List* keywords)
     Iterator* it;
     foreach (it, conditions) {
         Condition* cond = it->element;
+        if (strlen(cond->value) == 0) continue;
         sqlite3_bind_text(stmt, n++, cond->value, strlen(cond->value), NULL);
     }
     if (keywords->size > 0) {
@@ -433,7 +436,7 @@ int set_conditions(sqlite3_stmt* stmt, List* conditions, List* keywords)
         Iterator* it_keyword;
         Iterator* it;
         list_alloc(element_types_a, ElementType);
-        element_types_a= db_get_element_types_all(element_types_a);
+        element_types_a = db_get_element_types_all(element_types_a);
         foreach (it_keyword, keywords) {
             char* word = it_keyword->element;
             foreach (it, element_types_a) {
@@ -444,7 +447,7 @@ int set_conditions(sqlite3_stmt* stmt, List* conditions, List* keywords)
     }
     return n;
 }
-SearchResult* db_get_tickets_by_status(char* status, List* messages, SearchResult* result)
+SearchResult* db_get_tickets_by_status(const char* status, List* messages, SearchResult* result)
 {
     int r, n, hit_count = 0;
     String* sql_a = string_new(0);
@@ -488,7 +491,7 @@ SearchResult* db_get_tickets_by_status(char* status, List* messages, SearchResul
     return result;
 ERROR_LABEL
 }
-SearchResult* db_search_tickets(List* conditions, char* q, Condition* sorts, int page, List* messages, SearchResult* result)
+SearchResult* db_search_tickets(List* conditions, char* q, Condition* sorts, const int page, SearchResult* result)
 {
     int r, n;
     String* sql_a = string_new(0);
@@ -497,7 +500,6 @@ SearchResult* db_search_tickets(List* conditions, char* q, Condition* sorts, int
 
     list_alloc(keywords_a, char);
     keywords_a = parse_keywords(keywords_a, q);
-    result->messages = messages;
     sql_a = get_search_sql_string(conditions, sorts, keywords_a, sql_a);
 
     string_append(sql_a, " limit ? offset ? ");
@@ -520,15 +522,23 @@ SearchResult* db_search_tickets(List* conditions, char* q, Condition* sorts, int
     /* hit件数を取得する。 */
     {
         String* s = string_new(0);
-        string_append(s, "select count(*) from (");
+        string_append(s, "select count(res.id), res.state from (");
         s = get_search_sql_string(conditions, sorts, keywords_a, s);
-        string_append(s, ")");
+        string_appendf(s,
+                ") as res "
+                "inner join list_item as l on l.element_type_id = %d and l.name = res.state "
+                "group by res.state "
+                "order by l.sort", ELEM_ID_STATUS);
         if (sqlite3_prepare(db, string_rawstr(s), string_len(s), &stmt, NULL) == SQLITE_ERROR) goto error;
         sqlite3_reset(stmt);
         n = set_conditions(stmt, conditions, keywords_a);
         while (SQLITE_ROW == (r = sqlite3_step(stmt))){
-            result->hit_count = sqlite3_column_int(stmt, 0);
+            State* s = list_new_element(result->states);
+            result->hit_count += s->count = sqlite3_column_int(stmt, 0);
+            strcpy(s->name, sqlite3_column_text(stmt, 1));
+            list_add(result->states, s);
         }
+        string_free(s);
         if (SQLITE_DONE != r)
             goto error;
     }
@@ -590,7 +600,7 @@ static void set_str_val(Element* e, const unsigned char* str_val)
         strcpy(e->str_val, str_val);
     }
 }
-List* db_get_last_elements_4_list(int ticket_id, List* elements)
+List* db_get_last_elements_4_list(const int ticket_id, List* elements)
 {
     char sql[DEFAULT_LENGTH] = "";
     char sql_suf[DEFAULT_LENGTH] = "";
@@ -662,7 +672,7 @@ List* db_get_last_elements_4_list(int ticket_id, List* elements)
     return elements;
 ERROR_LABEL
 }
-List* db_get_last_elements(int ticket_id, List* elements)
+List* db_get_last_elements(const int ticket_id, List* elements)
 {
     char sql[DEFAULT_LENGTH];
     sqlite3_stmt *stmt = NULL;
@@ -773,7 +783,7 @@ List* db_get_elements(int message_id, List* elements)
 
 ERROR_LABEL
 }
-int* db_get_message_ids_a(int ticket_id)
+int* db_get_message_ids_a(const int ticket_id)
 {
     int* message_ids = NULL;
     int r, i = 0;
@@ -893,7 +903,7 @@ void db_update_list_item(ListItem* item)
             COLUMN_TYPE_END) != 1)
         die("no list_item to update?");
 }
-void db_delete_list_item(int id)
+void db_delete_list_item(const int id)
 {
     exec_query(
             "delete from list_item "
@@ -940,7 +950,7 @@ int db_register_element_type(ElementType* et)
     exec_query(sql, COLUMN_TYPE_END);
     return element_type_id;
 }
-void db_delete_element_type(int id)
+void db_delete_element_type(const int id)
 {
     exec_query(
             "update element_type set deleted = 1 where id = ?",
@@ -1014,7 +1024,7 @@ List* db_get_states(List* states)
 
 ERROR_LABEL
 }
-List* db_get_statictics_multi(List* states, int element_type_id)
+List* db_get_statictics_multi(List* states, const int element_type_id)
 {
     int r;
     char sql[DEFAULT_LENGTH];
@@ -1048,7 +1058,7 @@ List* db_get_statictics_multi(List* states, int element_type_id)
 
 ERROR_LABEL
 }
-List* db_get_statictics(List* states, int element_type_id)
+List* db_get_statictics(List* states, const int element_type_id)
 {
     int r;
     char sql[DEFAULT_LENGTH];
@@ -1120,7 +1130,7 @@ ElementFile* db_get_element_file(int id, ElementFile* file)
     return file;
 ERROR_LABEL
 }
-List* db_get_newest_information(int limit, List* messages)
+List* db_get_newest_information(const int limit, List* messages)
 {
     int r;
     char sql[DEFAULT_LENGTH];
@@ -1157,7 +1167,7 @@ int db_get_element_file_id(int message_id, int element_type_id)
             COLUMN_TYPE_INT, element_type_id,
             COLUMN_TYPE_END);
 }
-char* db_get_element_file_mime_type(int message_id, int element_type_id, char* buf)
+char* db_get_element_file_mime_type(const int message_id, const int element_type_id, char* buf)
 {
     int r;
     const char *sql;
