@@ -8,50 +8,65 @@
 #include "hook.h"
 
 unsigned long url_encode(unsigned char*, unsigned char*, unsigned long);
-static action* get_actions();
+static Action* get_actions();
 
-action* actions = NULL;
+Action* actions = NULL;
+int alloc_count = 0;
 
 void* xalloc(size_t size)
 {
     void* p;
     p = calloc(1, size);
     if (!p) {
-        d("memory error!!");
         die("memory error.");
     }
+    alloc_count++;
+/*     d("xalloc: %p\n", p); */
     return p;
 }
 void xfree(void* p)
 {
+/*     d("xfree: %p\n", p); */
+    alloc_count--;
     free(p);
 }
-static action* get_actions()
+static Action* get_actions()
 {
     return actions;
 }
 void register_action_actions(char* action_name, void func(void))
 {
-    action* a;
+    Action* a;
     if (actions == NULL) {
-        actions = xalloc(sizeof(action));
+        actions = xalloc(sizeof(Action));
         actions->action_name = "__HEAD__";
         actions->next = NULL;
     }
     for (a = actions; a->next != NULL; a = a->next);
-    a->next = xalloc(sizeof(action));
+    a->next = xalloc(sizeof(Action));
     a = a->next;
     a->action_name = action_name;
     a->action_func = func;
     a->next = NULL;
 }
+void free_action_actions()
+{
+    Action* a = actions->next;
+    while (a) {
+        Action* old = a;
+        a = a->next;
+        xfree(old);
+    }
+    xfree(actions);
+    d("alloc_count: %d\n", alloc_count);
+}
 
 void exec_action()
 {
     char* index;
     char* path_info = cgiPathInfo;
     char action_name[1024];
-    action* a;
+    Action* a;
     if (strlen(path_info) > 1) {
         strncpy(action_name, path_info + 1, 1024);
     } else {
@@ -70,7 +85,7 @@ void exec_action()
         }
     }
     if (strlen(action_name) != 0) {
-        /* 知らないactionが指定されたら、リダイレクトする。  */
+        /* 知らないActionが指定されたら、リダイレクトする。  */
         redirect("", NULL);
     } else {
         /* path_infoが空なら、top_actionを呼び出す。 */
@@ -94,7 +109,6 @@ int get_ticket_syntax_len(char* data, size_t len)
     if (len == 0) return ticket_syntax_len; /* 最後の文字だった */
     data += 1; /* 1文字進める。 */
     while (len--) {
-        d("%c\n", *data);
         if (*data < '0' || *data > '9') 
             break; /* 数字でない場合は、breakする。 */
         data++;
@@ -108,7 +122,7 @@ int get_ticket_syntax_len(char* data, size_t len)
  */
 static cgiFormResultType cgiHtmlEscapeDataMultiLine(char *data, int len)
 {
-    int printing_pre = 0;
+    bool printing_pre = false;
     while (len--) {
         if (*data == '#') {
             int ticket_syntax_len;
@@ -134,7 +148,7 @@ static cgiFormResultType cgiHtmlEscapeDataMultiLine(char *data, int len)
             TRYPUTC('>');
             data += 2;
             len -= 2;
-            printing_pre = 1;
+            printing_pre = true;
         } else if (len > 0 && *data == '|' &&
                 *(data + 1) == '<' &&
                 (*(data + 2) == '\r' || *(data + 2) == '\n' || len == 1)) {
@@ -147,7 +161,7 @@ static cgiFormResultType cgiHtmlEscapeDataMultiLine(char *data, int len)
             TRYPUTC('>');
             data += (len == 1) ? 1 : 2; /* |< で終わっている場合(改行が無い場合) は、1を足す。改行が付いている場合は、2を足す。*/
             len -= (len == 1) ? 1 : 2;
-            printing_pre = 0;
+            printing_pre = false;
         } else if (*data == '<') {
             TRYPUTC('&');
             TRYPUTC('l');
@@ -164,7 +178,7 @@ static cgiFormResultType cgiHtmlEscapeDataMultiLine(char *data, int len)
             TRYPUTC('g');
             TRYPUTC('t');
             TRYPUTC(';');
-        } else if (printing_pre == 0 && *data == '\n') {
+        } else if (printing_pre == false && *data == '\n') {
             TRYPUTC('<');
             TRYPUTC('b');
             TRYPUTC('r');
@@ -230,50 +244,50 @@ void u(char* str)
 }
 unsigned long url_encode(unsigned char* csource, unsigned char* cbuffer, unsigned long lbuffersize)
 {
-    unsigned long   llength;                                        /* csource のサイズを格納 */
-    unsigned long   lcount = 0;                                     /* csource の読み込み位置カウンタ */
-    unsigned char   cbyte;                                          /* 抜き出された 1 バイト分のデータを格納 */
-    unsigned char   ctemp[4];                                       /* 変換結果(1 文字分)一時格納バッファ */
-    unsigned long   lresultcount = 0;                               /* cbuffer の書き込み位置カウンタ */
+    unsigned long llength;
+    unsigned long lcount = 0;
+    unsigned char cbyte;
+    unsigned char ctemp[4];
+    unsigned long lresultcount = 0;
 
-    llength = (unsigned long)strlen(csource);                       /* csource の文字サイズを得る */
-    if(!llength) { return lresultcount; }                           /* csource が 0 文字の場合、関数を抜ける */
-    if(lbuffersize < (llength * 3 + 1)) { return lresultcount; }    /* バッファサイズが足りない場合、関数を抜ける */
+    llength = (unsigned long)strlen(csource);
+    if(!llength) { return lresultcount; }
+    if(lbuffersize < (llength * 3 + 1)) { return lresultcount; }
 
     while(1) {
-        cbyte = *(csource + lcount);                                /* 1 バイトを抜き出す */
+        cbyte = *(csource + lcount);
         if( ((cbyte >= 0x81) && (cbyte <= 0x9F)) ||
-                ((cbyte >= 0xE0) && (cbyte <= 0xEF)) ) {                /* Shift-JIS 2 バイト文字だった場合 */
+                ((cbyte >= 0xE0) && (cbyte <= 0xEF)) ) {            /* Shift-JIS 2 バイト文字だった場合 */
             sprintf(ctemp, "%%%02X", cbyte);                        /* URL エンコード(上位バイト) */
-            strncpy(cbuffer + lresultcount, ctemp, 4);              /* cbuffer にコピー */
-            lcount++;                                               /* 読み込みカウンタをインクリメント */
-            lresultcount += 3;                                      /* 書き込みカウンタを 3 増やす */
+            strncpy(cbuffer + lresultcount, ctemp, 4);
+            lcount++;
+            lresultcount += 3;
             if(lcount == llength) { break; }                        /* 文字列の終端に達した場合、ループを抜ける */
             sprintf(ctemp, "%%%02X", *(csource + lcount));          /* URL エンコード(下位バイト) */
-            strncpy(cbuffer + lresultcount, ctemp, 4);              /* cbuffer にコピー */
-            lcount++;                                               /* 読み込みカウンタをインクリメント */
-            lresultcount += 3;                                      /* 書き込みカウンタを 3 増やす */
+            strncpy(cbuffer + lresultcount, ctemp, 4);
+            lcount++;
+            lresultcount += 3;
         } else if(cbyte == 0x20) {                                  /* 1 バイト半角スペース(" ")だった場合 */
-            strncpy(cbuffer + lresultcount, "+", 2);                /* "+" を cbuffer にコピー */
-            lcount++;                                               /* 読み込みカウンタをインクリメント */
-            lresultcount++;                                         /* 書き込みカウンタをインクリメント */
+            strncpy(cbuffer + lresultcount, "+", 2);
+            lcount++;
+            lresultcount++;
         } else if( ((cbyte >= 0x40) && (cbyte <= 0x5A)) ||          /* @A-Z */
-                ((cbyte >= 0x61) && (cbyte <= 0x7A)) ||          /* a-z */
-                ((cbyte >= 0x30) && (cbyte <= 0x39)) ||          /* 0-9 */
-                (cbyte == 0x2A) ||                               /* "*" */
-                (cbyte == 0x2D) ||                               /* "-" */
-                (cbyte == 0x2E) ||                               /* "." */
-                (cbyte == 0x5F) ) {                              /* "_" */ /* 無変換文字だった場合 */
-            strncpy(cbuffer + lresultcount, csource + lcount, 2);   /* そのまま cbuffer にコピー */
-            lcount++;                                               /* 読み込みカウンタをインクリメント */
-            lresultcount++;                                         /* 書き込みカウンタをインクリメント */
+                ((cbyte >= 0x61) && (cbyte <= 0x7A)) ||             /* a-z */
+                ((cbyte >= 0x30) && (cbyte <= 0x39)) ||             /* 0-9 */
+                (cbyte == 0x2A) ||                                  /* "*" */
+                (cbyte == 0x2D) ||                                  /* "-" */
+                (cbyte == 0x2E) ||                                  /* "." */
+                (cbyte == 0x5F) ) {                                 /* "_" */ /* 無変換文字だった場合 */
+            strncpy(cbuffer + lresultcount, csource + lcount, 2);
+            lcount++;
+            lresultcount++;
         } else {                                                    /* その他の文字の場合 */
             sprintf(ctemp, "%%%02X", cbyte);                        /* URL エンコード */
-            strncpy(cbuffer + lresultcount, ctemp, 4);              /* cbuffer にコピー */
-            lcount++;                                               /* 読み込みカウンタをインクリメント */
-            lresultcount += 3;                                      /* 書き込みカウンタを 3 増やす */
+            strncpy(cbuffer + lresultcount, ctemp, 4);
+            lcount++;
+            lresultcount += 3;
         }
-        if(lcount == llength) { break; }                            /* 文字列の終端に達した場合、ループを抜ける */
+        if(lcount == llength) { break; }
     }
     return lresultcount;                                            /* cbuffer に書き込んだ文字列のサイズを返す */
 }
@@ -290,14 +304,14 @@ char* get_filename_without_path(char* path)
     }
     return p;
 }
-char* get_upload_filename(int element_id, char* buf)
+char* get_upload_filename(const int element_id, char* buf)
 {
     char name[DEFAULT_LENGTH];
     sprintf(name, "field%d", element_id);
     cgiFormFileName(name, buf, DEFAULT_LENGTH);
     return buf;
 }
-int get_upload_size(int element_id)
+int get_upload_size(const int element_id)
 {
     char name[DEFAULT_LENGTH];
     int size;
@@ -305,22 +319,22 @@ int get_upload_size(int element_id)
     cgiFormFileSize(name, &size);
     return size;
 }
-char* get_upload_content_type(int element_id, char* buf)
+char* get_upload_content_type(const int element_id, char* buf)
 {
     char name[DEFAULT_LENGTH];
     sprintf(name, "field%d", element_id);
     cgiFormFileContentType(name, buf, DEFAULT_LENGTH);
     return buf;
 }
-ElementFile* get_upload_content(int element_id)
+ElementFile* get_upload_content(const int element_id)
 {
-    int got = 0;
+    int got_count = 0;
     char* buffer_org;
     char* buffer;
     char b[DEFAULT_LENGTH];
     char name[DEFAULT_LENGTH];
     cgiFilePtr file;
-    ElementFile* content = xalloc(sizeof(ElementFile));
+    ElementFile* content = element_file_new();
     content->size = get_upload_size(element_id);
     buffer = buffer_org = xalloc(sizeof(char) * content->size);
     sprintf(name, "field%d", element_id);
@@ -328,10 +342,10 @@ ElementFile* get_upload_content(int element_id)
         die("Could not open the file.");
     }
 
-    while (cgiFormFileRead(file, b, sizeof(b), &got) == cgiFormSuccess) {
+    while (cgiFormFileRead(file, b, sizeof(b), &got_count) == cgiFormSuccess) {
         char* p = b;
         int i;
-        for (i = 0; i < got; i++) {
+        for (i = 0; i < got_count; i++) {
             *buffer = *p;
             buffer++;
             p++;
@@ -380,7 +394,7 @@ void base64_encode(const unsigned char *src, unsigned char *dist)
     }
     if (i) enclode_char(bb, i - 1, dist, j);
 }
-void redirect(char* path, char* message)
+void redirect(const char* path, const char* message)
 {
     char redirecturi[DEFAULT_LENGTH];
     char uri[DEFAULT_LENGTH];
@@ -398,7 +412,7 @@ void redirect(char* path, char* message)
     o("Status: 302 Temporary Redirection\r\n");
     cgiHeaderLocation(redirecturi);
 }
-void redirect_with_hook_messages(char* path, char* message, List* results)
+void redirect_with_hook_messages(const char* path, const char* message, List* results)
 {
     char redirecturi[DEFAULT_LENGTH];
     char uri[DEFAULT_LENGTH];
@@ -432,15 +446,6 @@ void redirect_with_hook_messages(char* path, char* message, List* results)
     o("Status: 302 Temporary Redirection\r\n");
     cgiHeaderLocation(redirecturi);
 }
-void free_element_list(List* elements)
-{
-    Iterator* it;
-    foreach (it, elements) {
-        Element* e = it->element;
-        xfree(e->str_val);
-    }
-    list_free(elements);
-}
 static cgiFormResultType csv_escape(char *data, int len)
 {
     while (len--) {
@@ -515,4 +520,23 @@ String* get_base_url(String* buf)
             cgiScriptName);
     return buf;
 }
+bool contains(char* const value, const char* name)
+{
+    char* p = value;
+    do {
+        if (*p == '\t') p++;
+        if (strncmp(p, name, strlen(name)) == 0)
+            return true;
+    } while ((p = strstr(p, "\t")) != NULL);
+    return false;
+}
+void set_cookie(char* key, char* value)
+{
+    cgiHeaderCookieSetString(key, value, 86400 * 30, "/", cgiServerName);
+}
+void clear_cookie(char* key)
+{
+    cgiHeaderCookieSetString(key, "", 0, "/", cgiServerName);
+}
+
 /* vim: set ts=4 sw=4 sts=4 expandtab fenc=utf-8: */
