@@ -16,25 +16,28 @@ typedef struct _action {
     char* action_name;
     void (*action_func)(void);
     struct _action* next;
-} action;
+} Action;
 void register_action_actions(char*, void func(void));
+void free_action_actions();
 void exec_action();
 void hm(char*);
 void hmail(char*);
 char* get_filename_without_path(char*);
-char* get_upload_filename(int, char*);
-int get_upload_size(int);
-char* get_upload_content_type(int, char*);
-ElementFile* get_upload_content(int);
+char* get_upload_filename(const int, char*);
+int get_upload_size(const int);
+char* get_upload_content_type(const int, char*);
+ElementFile* get_upload_content(const int);
 unsigned long url_encode(unsigned char*, unsigned char*, unsigned long);
-void redirect(char*, char*);
-void redirect_with_hook_messages(char*, char*, List*);
-void free_element_list(List*);
+void redirect(const char*, const char*);
+void redirect_with_hook_messages(const char*, const char*, List*);
 void* xalloc(size_t);
 void xfree(void*);
 void csv_field(char*);
 void css_field(char*);
 String* get_base_url(String*);
+bool contains(char* const, const char*);
+void set_cookie(char*, char*);
+void clear_cookie(char*);
 
 #endif
 /* vim: set ts=4 sw=4 sts=4 expandtab fenc=utf-8: */
